@@ -8,7 +8,6 @@
 #ifndef LDNS_DNSSEC_ZONE_H
 #define LDNS_DNSSEC_ZONE_H
  
-#include <ldns/ldns.h>
 #include <ldns/rbtree.h>
 #include <ldns/host2str.h>
 
@@ -94,6 +93,13 @@ struct ldns_struct_dnssec_zone {
 	ldns_dnssec_name *soa;
 	/** tree of ldns_dnssec_names */
 	ldns_rbtree_t *names;
+	/** tree of ldns_dnssec_names by nsec3 hashes (when applicable) */
+	ldns_rbtree_t *hashed_names;
+	/** points to the first added NSEC3 rr whose parameters will be 
+	 *  assumed for all subsequent NSEC3 rr's and which will be used
+	 *  to calculate hashed names
+	 */
+	ldns_rr *_nsec3params;
 };
 typedef struct ldns_struct_dnssec_zone ldns_dnssec_zone;
 
@@ -101,7 +107,7 @@ typedef struct ldns_struct_dnssec_zone ldns_dnssec_zone;
  * Creates a new entry for 1 pointer to an rr and 1 pointer to the next rrs
  * \return the allocated data
  */
-ldns_dnssec_rrs *ldns_dnssec_rrs_new();
+ldns_dnssec_rrs *ldns_dnssec_rrs_new(void);
 
 /**
  * Frees the list of rrs, but *not* the individual ldns_rr records
@@ -120,7 +126,8 @@ void ldns_dnssec_rrs_free(ldns_dnssec_rrs *rrs);
 void ldns_dnssec_rrs_deep_free(ldns_dnssec_rrs *rrs);
 
 /**
- * Adds an RR to the list of RRs. The list will remain ordered
+ * Adds an RR to the list of RRs. The list will remain ordered.
+ * If an equal RR already exists, this RR will not be added.
  *
  * \param[in] rrs the list to add to
  * \param[in] rr the RR to add
@@ -134,7 +141,7 @@ ldns_status ldns_dnssec_rrs_add_rr(ldns_dnssec_rrs *rrs, ldns_rr *rr);
  * \param[in] out the file descriptor to print to
  * \param[in] rrs the list of RRs to print
  */
-void ldns_dnssec_rrs_print(FILE *out, ldns_dnssec_rrs *rrs);
+void ldns_dnssec_rrs_print(FILE *out, const ldns_dnssec_rrs *rrs);
 
 /**
  * Prints the given rrs to the file descriptor
@@ -144,13 +151,13 @@ void ldns_dnssec_rrs_print(FILE *out, ldns_dnssec_rrs *rrs);
  * \param[in] rrs the list of RRs to print
  */
 void ldns_dnssec_rrs_print_fmt(FILE *out, 
-		const ldns_output_format *fmt, ldns_dnssec_rrs *rrs);
+		const ldns_output_format *fmt, const ldns_dnssec_rrs *rrs);
 
 /**
  * Creates a new list (entry) of RRsets
  * \return the newly allocated structure
  */
-ldns_dnssec_rrsets *ldns_dnssec_rrsets_new();
+ldns_dnssec_rrsets *ldns_dnssec_rrsets_new(void);
 
 /**
  * Frees the list of rrsets and their rrs, but *not* the ldns_rr
@@ -174,7 +181,7 @@ void ldns_dnssec_rrsets_deep_free(ldns_dnssec_rrsets *rrsets);
  * \param[in] rrsets the rrset to get the type of
  * \return the rr type
  */
-ldns_rr_type ldns_dnssec_rrsets_type(ldns_dnssec_rrsets *rrsets);
+ldns_rr_type ldns_dnssec_rrsets_type(const ldns_dnssec_rrsets *rrsets);
 
 /**
  * Sets the RR type of the rrset (that is head of the given list)
@@ -197,18 +204,18 @@ ldns_status ldns_dnssec_rrsets_set_type(ldns_dnssec_rrsets *rrsets,
 ldns_status ldns_dnssec_rrsets_add_rr(ldns_dnssec_rrsets *rrsets, ldns_rr *rr);
 
 /**
- * Print the given list of rrsets to the fiven file descriptor
+ * Print the given list of rrsets to the given file descriptor
  * 
  * \param[in] out the file descriptor to print to
  * \param[in] rrsets the list of RRsets to print
  * \param[in] follow if set to false, only print the first RRset
  */ 
 void ldns_dnssec_rrsets_print(FILE *out,
-		ldns_dnssec_rrsets *rrsets,
+		const ldns_dnssec_rrsets *rrsets,
 		bool follow);
 
 /**
- * Print the given list of rrsets to the fiven file descriptor
+ * Print the given list of rrsets to the given file descriptor
  * 
  * \param[in] out the file descriptor to print to
  * \param[in] fmt the format of the textual representation
@@ -217,7 +224,7 @@ void ldns_dnssec_rrsets_print(FILE *out,
  */ 
 void ldns_dnssec_rrsets_print_fmt(FILE *out,
 		const ldns_output_format *fmt,
-		ldns_dnssec_rrsets *rrsets,
+		const ldns_dnssec_rrsets *rrsets,
 		bool follow);
 
 
@@ -225,7 +232,7 @@ void ldns_dnssec_rrsets_print_fmt(FILE *out,
  * Create a new data structure for a dnssec name
  * \return the allocated structure
  */
-ldns_dnssec_name *ldns_dnssec_name_new();
+ldns_dnssec_name *ldns_dnssec_name_new(void);
 
 /**
  * Create a new data structure for a dnssec name for the given RR
@@ -256,7 +263,7 @@ void ldns_dnssec_name_deep_free(ldns_dnssec_name *name);
  * \param[in] name the dnssec name to get the domain name from
  * \return the domain name
  */
-ldns_rdf *ldns_dnssec_name_name(ldns_dnssec_name *name);
+ldns_rdf *ldns_dnssec_name_name(const ldns_dnssec_name *name);
 
 
 /**
@@ -278,7 +285,7 @@ void ldns_dnssec_name_set_name(ldns_dnssec_name *name,
  * \param[in] name the dnssec name to get the domain name from
  * \return true if the structure is marked as glue, false otherwise.
  */
-bool ldns_dnssec_name_is_glue(ldns_dnssec_name *name);
+bool ldns_dnssec_name_is_glue(const ldns_dnssec_name *name);
 
 /**
  * Sets the NSEC(3) RR of the given dnssec_name structure
@@ -290,12 +297,12 @@ void ldns_dnssec_name_set_nsec(ldns_dnssec_name *name, ldns_rr *nsec);
 
 /**
  * Compares the domain names of the two arguments in their
- * canonical ordening.
+ * canonical ordering.
  * 
  * \param[in] a The first dnssec_name to compare
  * \param[in] b The second dnssec_name to compare
  * \return -1 if the domain name of a comes before that of b in canonical
- *            ordening, 1 if it is the other way around, and 0 if they are
+ *            ordering, 1 if it is the other way around, and 0 if they are
  *            equal
  */
 int ldns_dnssec_name_cmp(const void *a, const void *b);
@@ -318,7 +325,7 @@ ldns_status ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
  * \param[in] type the type of the RRset to find
  * \return the RRset, or NULL if not present
  */
-ldns_dnssec_rrsets *ldns_dnssec_name_find_rrset(ldns_dnssec_name *name,
+ldns_dnssec_rrsets *ldns_dnssec_name_find_rrset(const ldns_dnssec_name *name,
 									   ldns_rr_type type);
 
 /**
@@ -329,8 +336,8 @@ ldns_dnssec_rrsets *ldns_dnssec_name_find_rrset(ldns_dnssec_name *name,
  * \param[in] type the type of the RRset to find
  * \return the RRset, or NULL if not present
  */
-ldns_dnssec_rrsets *ldns_dnssec_zone_find_rrset(ldns_dnssec_zone *zone,
-									   ldns_rdf *dname,
+ldns_dnssec_rrsets *ldns_dnssec_zone_find_rrset(const ldns_dnssec_zone *zone,
+									   const ldns_rdf *dname,
 									   ldns_rr_type type);
 
 /**
@@ -340,7 +347,7 @@ ldns_dnssec_rrsets *ldns_dnssec_zone_find_rrset(ldns_dnssec_zone *zone,
  * \param[in] out the file descriptor to print to
  * \param[in] name the name structure to print the contents of
  */
-void ldns_dnssec_name_print(FILE *out, ldns_dnssec_name *name);
+void ldns_dnssec_name_print(FILE *out, const ldns_dnssec_name *name);
 
 /**
  * Prints the RRs in the  dnssec name structure to the given
@@ -351,13 +358,40 @@ void ldns_dnssec_name_print(FILE *out, ldns_dnssec_name *name);
  * \param[in] name the name structure to print the contents of
  */
 void ldns_dnssec_name_print_fmt(FILE *out, 
-		const ldns_output_format *fmt, ldns_dnssec_name *name);
+		const ldns_output_format *fmt, const ldns_dnssec_name *name);
 
 /**
  * Creates a new dnssec_zone structure
  * \return the allocated structure
  */
-ldns_dnssec_zone *ldns_dnssec_zone_new();
+ldns_dnssec_zone *ldns_dnssec_zone_new(void);
+
+/**
+ * Create a new dnssec zone from a file.
+ * \param[out] z the new zone
+ * \param[in] *fp the filepointer to use
+ * \param[in] *origin the zones' origin
+ * \param[in] c default class to use (IN)
+ * \param[in] ttl default ttl to use
+ *
+ * \return ldns_status mesg with an error or LDNS_STATUS_OK
+ */
+ldns_status ldns_dnssec_zone_new_frm_fp(ldns_dnssec_zone** z, FILE* fp,
+		const ldns_rdf* origin, uint32_t ttl, ldns_rr_class c);
+
+/**
+ * Create a new dnssec zone from a file, keep track of the line numbering
+ * \param[out] z the new zone
+ * \param[in] *fp the filepointer to use
+ * \param[in] *origin the zones' origin
+ * \param[in] ttl default ttl to use
+ * \param[in] c default class to use (IN)
+ * \param[out] line_nr used for error msg, to get to the line number
+ *
+ * \return ldns_status mesg with an error or LDNS_STATUS_OK
+ */
+ldns_status ldns_dnssec_zone_new_frm_fp_l(ldns_dnssec_zone** z, FILE* fp,
+		const ldns_rdf* origin, uint32_t ttl, ldns_rr_class c, int* line_nr);
 
 /**
  * Frees the given zone structure, and its rbtree of dnssec_names
@@ -393,7 +427,7 @@ ldns_status ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone,
  * \param[in] tree the tree of ldns_dnssec_name structures to print
  * \param[in] print_soa if true, print SOA records, if false, skip them
  */
-void ldns_dnssec_zone_names_print(FILE *out, ldns_rbtree_t *tree, bool print_soa);
+void ldns_dnssec_zone_names_print(FILE *out, const ldns_rbtree_t *tree, bool print_soa);
 
 /**
  * Prints the rbtree of ldns_dnssec_name structures to the file descriptor
@@ -404,7 +438,7 @@ void ldns_dnssec_zone_names_print(FILE *out, ldns_rbtree_t *tree, bool print_soa
  * \param[in] print_soa if true, print SOA records, if false, skip them
  */
 void ldns_dnssec_zone_names_print_fmt(FILE *out, const ldns_output_format *fmt,
-		ldns_rbtree_t *tree, bool print_soa);
+		const ldns_rbtree_t *tree, bool print_soa);
 
 /**
  * Prints the complete zone to the given file descriptor
@@ -412,7 +446,7 @@ void ldns_dnssec_zone_names_print_fmt(FILE *out, const ldns_output_format *fmt,
  * \param[in] out the file descriptor to print to
  * \param[in] zone the dnssec_zone to print
  */
-void ldns_dnssec_zone_print(FILE *out, ldns_dnssec_zone *zone);
+void ldns_dnssec_zone_print(FILE *out, const ldns_dnssec_zone *zone);
 
 /**
  * Prints the complete zone to the given file descriptor
@@ -422,7 +456,7 @@ void ldns_dnssec_zone_print(FILE *out, ldns_dnssec_zone *zone);
  * \param[in] zone the dnssec_zone to print
  */
 void ldns_dnssec_zone_print_fmt(FILE *out, 
-		const ldns_output_format *fmt, ldns_dnssec_zone *zone);
+		const ldns_output_format *fmt, const ldns_dnssec_zone *zone);
 
 /**
  * Adds explicit dnssec_name structures for the empty nonterminals
@@ -433,6 +467,17 @@ void ldns_dnssec_zone_print_fmt(FILE *out,
  */
 ldns_status ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone);
 
+/**
+ * If a NSEC3PARAM is available in the apex, walks the zone and returns true
+ * on the first optout nsec3.
+ *
+ * \param[in] zone the zone to check for nsec3 optout records
+ * return true when the zone has at least one nsec3 optout record.
+ */
+bool ldns_dnssec_zone_is_nsec3_optout(const ldns_dnssec_zone* zone);
+
+ldns_status ldns_dnssec_zone_verify_zonemd(ldns_dnssec_zone *zone);
+
 #ifdef __cplusplus
 }
 #endif
